USE [master]
GO

-- Drop if already exist
DROP DATABASE IF EXISTS [TempDBDemo];
GO

-- Create new DB
CREATE DATABASE [TempDBDemo];
GO


-- Create sp_TempDBStressTest
USE [TempDBDemo];
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- =============================================
-- Author:		Vitaly @Madeira
-- Create date: 2020-01-12 
-- Description:	TempDB Stress test
-- =============================================
CREATE OR ALTER PROCEDURE [sp_TempDBStressTest]
AS
BEGIN

	SET NOCOUNT ON;

	-- check if Demo DB already exist
	DROP TABLE IF EXISTS #RandomValues;

	-- Create table random values
	CREATE TABLE #RandomValues
						(
							Id				INT IDENTITY(1,1),
							[Name]			UNIQUEIDENTIFIER		DEFAULT NEWID(),
							InsertDate		DATETIME				DEFAULT GETDATE()
						);

	-- Populate table
	INSERT INTO #RandomValues
	DEFAULT VALUES;

	-- Update rows
	UPDATE t
	SET
		t.InsertDate = DATEADD(DAY, -1, t.InsertDate)
	FROM
		#RandomValues t;

	-- Delete rows
	DELETE
	FROM
		#RandomValues
	WHERE
		InsertDate < GETDATE();

END
GO

-- Clean 
DBCC FREEPROCCACHE
GO

-- run stress  start > RML CMD Prompt
-- ostress -E -Q"EXEC TempDBDemo..sp_TempDBStressTest " -n50 -r1000





-- Add a few files to tempDB
--USE [master]
--GO
--ALTER DATABASE [tempdb] ADD FILE ( NAME = N'tempdev2', FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL15.MSSQLSERVER\MSSQL\DATA\tempdev2.ndf' , SIZE = 8192KB , FILEGROWTH = 1024KB )
--GO
--ALTER DATABASE [tempdb] ADD FILE ( NAME = N'tempdev3', FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL15.MSSQLSERVER\MSSQL\DATA\tempdev3.ndf' , SIZE = 8192KB , FILEGROWTH = 1024KB )
--GO
--ALTER DATABASE [tempdb] ADD FILE ( NAME = N'tempdev4', FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL15.MSSQLSERVER\MSSQL\DATA\tempdev4.ndf' , SIZE = 8192KB , FILEGROWTH = 1024KB )
--GO
--ALTER DATABASE [tempdb] ADD FILE ( NAME = N'tempdev5', FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL15.MSSQLSERVER\MSSQL\DATA\tempdev5.ndf' , SIZE = 8192KB , FILEGROWTH = 1024KB )
--GO
--ALTER DATABASE [tempdb] ADD FILE ( NAME = N'tempdev6', FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL15.MSSQLSERVER\MSSQL\DATA\tempdev6.ndf' , SIZE = 8192KB , FILEGROWTH = 1024KB )
--GO
--ALTER DATABASE [tempdb] ADD FILE ( NAME = N'tempdev7', FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL15.MSSQLSERVER\MSSQL\DATA\tempdev7.ndf' , SIZE = 8192KB , FILEGROWTH = 1024KB )
--GO
--ALTER DATABASE [tempdb] ADD FILE ( NAME = N'tempdev8', FILENAME = N'C:\Program Files\Microsoft SQL Server\MSSQL15.MSSQLSERVER\MSSQL\DATA\tempdev8.ndf' , SIZE = 8192KB , FILEGROWTH = 1024KB )
--GO




-- The counters are updated only when a task ends; they do not reflect running tasks.
SELECT *
FROM
	sys.dm_db_session_space_usage;
GO

SELECT *
FROM
	sys.dm_db_task_space_usage;
GO