USE [master]
GO

SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

;WITH
	jobs_CTE
AS
	(
		SELECT
			N'SQLAgent - TSQL JobStep (Job 0x'+ CONVERT(CHAR(32),CAST(j.job_id AS BINARY(16)),2) + N' : Step ' + CAST(js.step_id AS NVARCHAR(3)) + N')' AS [ProgramName]
			,j.[name]
			,js.step_name
		FROM
			msdb.dbo.sysjobs AS j 
			INNER JOIN msdb.dbo.sysjobsteps AS js ON j.job_id = js.job_id
	),
	exec_requests_CTE
AS
	(
		SELECT
			er.session_id,
			er.blocking_session_id,
			es.[Status],
			es.last_request_end_time,
			er.Wait_Time,
			ISNULL(er.Wait_Type,N'')	AS Wait_Type,
			er.wait_resource,
			er.last_wait_type,
			er.Cpu_Time,
			er.Logical_Reads,
			er.Reads,
			er.Writes,
			er.database_id,
			er.statement_start_offset,
			er.statement_end_offset,
			es.login_name,
			es.original_login_name,
			es.[Host_Name],
			er.Percent_Complete,
			er.Estimated_Completion_Time,
			er.[sql_handle],
			er.plan_handle,
			er.connection_id,
			es.client_interface_name,
			CASE
				WHEN (es.[Program_Name] = es.client_interface_name) OR (es.client_interface_name IS NULL) THEN
																												(
																													CASE
																														WHEN es.[Program_Name] LIKE N'%Operating System%'						THEN REPLACE(es.[Program_Name], N'Operating System', N'OS')
																														WHEN es.[Program_Name] LIKE N'Microsoft SQL Server Management Studio%'	THEN REPLACE(es.[Program_Name], N'Microsoft SQL Server Management Studio', N'SSMS')
																														WHEN es.[Program_Name] LIKE N'SQLAgent - TSQL JobStep%'					THEN
																																																	(
																																																		SELECT
																																																			N'Job: ' + name + N', Step: ' + step_name
																																																		FROM
																																																			jobs_CTE jb
																																																		WHERE
																																																			jb.[ProgramName] = es.[Program_Name]
																																																	)
																														ELSE es.[Program_Name]
																													END
																												)
				WHEN (es.[Program_Name] IS NULL) AND (es.client_interface_name IS NOT NULL) THEN es.client_interface_name
				ELSE es.client_interface_name + N' - ' + (
															CASE
																WHEN es.[Program_Name] LIKE N'%Operating System%'						THEN REPLACE(es.[Program_Name], N'Operating System', N'OS')
																WHEN es.[Program_Name] LIKE N'Microsoft SQL Server Management Studio%'	THEN REPLACE(es.[Program_Name], N'Microsoft SQL Server Management Studio', N'SSMS')
																WHEN es.[Program_Name] LIKE N'SQLAgent - TSQL JobStep%'					THEN
																																				(
																																					SELECT
																																						N'SQLAgent - Job: ' + name + N', Step: ' + step_name
																																					FROM
																																						jobs_CTE jb
																																					WHERE jb.[ProgramName] = es.[Program_Name]
																																				)
																ELSE es.[Program_Name]
															END
														)
			END																		AS [Client & Program]
		FROM
			sys.dm_exec_requests er
			LEFT JOIN	sys.dm_exec_sessions es				ON es.session_id = er.session_id
		WHERE
			es.[status] = N'running'
			AND
			er.session_id <> @@SPID
		)
SELECT
	er.session_id																		AS SPID,					-- SQL Server session ID
	er.blocking_session_id																AS [Blk by],
	er.[Status]																			AS [Status],
	DATEDIFF(SECOND, er.last_request_end_time, GETDATE())								AS [Sec],
	er.Wait_Time																		AS [Wait (ms)],				-- If the request is currently blocked, this column returns the duration in milliseconds, of the current wait. Is not nullable.
	er.Wait_Type																		AS [Wait Type],				-- If the request is currently blocked, this column returns the type of wait. Is nullable.
	er.last_wait_type																	AS [Last Wait Type],		-- If this request has previously been blocked, this column returns the type of the last wait. Is not nullable.
	er.wait_resource																	AS [Wait Resource],			-- Textual representation of a lock resource.	
	--CASE 
	--	WHEN dtl.resource_type = 'OBJECT' THEN OBJECT_NAME(dtl.resource_associated_entity_id)
	--	ELSE OBJECT_NAME(pt.[object_id])
	--END																					AS [Locked Object],
	(SELECT TOP 1 ISNULL(sp.ecid,0) + 1 FROM sys.sysprocesses sp WHERE sp.spid = er.session_id ORDER BY ecid DESC)	AS Threads,
	CAST(mg.query_cost AS DECIMAL(10, 2))												AS Cost,					-- Estimated query cost.
	er.Cpu_Time																			AS [CPU (ms)],				-- CPU time in milliseconds that is used by the request. Is not nullable.
	er.Logical_Reads																	AS [L.Reads],				-- Number of logical reads that have been performed by the request. Is not nullable.
	er.Reads																			AS [P.Reads],				-- Number of reads performed by this request. Is not nullable.
	er.Writes																			AS Writes,					-- Number of writes performed by this request. Is not nullable.
	mg.requested_memory_kb/1024															AS [R.Mem (MB)],			-- Total requested amount of memory in megabytes.
	mg.granted_memory_kb/1024															AS [G.Mem (MB)],			-- Total amount of memory actually granted in megabytes. Can be NULL if the memory is not granted yet. For a typical situation, this value should be the same as requested_memory_kb. For index creation, the server may allow additional on-demand memory beyond initially granted memory.
	(mg.granted_memory_kb - mg.ideal_memory_kb)/1024									AS [M.Mem (MB)],			-- Missing amount of memory in megabytes (Total amount of memory actually granted minus size of the memory grant to fit everything into physical memory, based on the cardinality estimate).
	OBJECT_NAME(st.objectid, er.database_id)											AS [Object],
	SUBSTRING(st.[text], (er.statement_start_offset/2)+1,((CASE er.statement_end_offset WHEN -1 THEN DATALENGTH(st.[text]) WHEN 0 THEN DATALENGTH(st.[text]) ELSE er.statement_end_offset END - er.statement_start_offset)/2)+1) [Statement],
	CAST(p.Query_Plan AS XML)															AS [Plan],
	ISNULL(OBJECT_SCHEMA_NAME(st.objectid, er.database_id),N'dbo')						AS [Schema],
	DB_NAME(er.database_id)																AS [Database],
	CASE
		WHEN er.Login_Name	= N'' THEN er.original_login_name
		ELSE er.Login_Name
	END																					AS [Login],
	CASE
		WHEN ec.client_tcp_port IS NULL THEN ec.client_net_address
		ELSE ec.client_net_address + N':'+ CONVERT(NVARCHAR, ec.client_tcp_port,0)
	END																					AS [Client Net Address],
	er.[Host_Name]																		AS Host,
	er.[Client & Program],
	er.Percent_Complete																	AS [%_Complete]
FROM
	exec_requests_CTE er
	OUTER APPLY sys.dm_exec_sql_text(er.[sql_handle]) st	
	OUTER APPLY sys.dm_exec_text_query_plan(er.plan_handle, er.statement_start_offset, er.statement_end_offset) p
	LEFT JOIN	sys.dm_exec_query_memory_grants mg	ON mg.session_id = er.session_id AND er.[sql_handle] = mg.[sql_handle]
	LEFT JOIN	sys.dm_exec_connections ec			ON ec.session_id = er.session_id AND ec.connection_id = er.connection_id
	--LEFT JOIN	sys.dm_tran_locks dtl ON er.session_id = dtl.request_session_id
	--LEFT JOIN	sys.partitions AS pt ON dtl.resource_associated_entity_id = pt.hobt_id
ORDER BY
	SPID ASC,
	Threads DESC,
	Sec DESC
OPTION (RECOMPILE, MAXDOP 1); 



